﻿Type=Class
Version=5.02
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
'Class module
Sub Class_Globals
	Private mAccessToken As String	
	Private mApiV1url As String = "https://api.dropbox.com/1/"
	Private mApiV1urlcontent As String = "https://api-content.dropbox.com/1/"
	Private mApiV2url As String = "https://api.dropboxapi.com/2-beta-2/"
	Private mApiV2urlContent As String = "https://content.dropboxapi.com/2-beta-2/"
	Private mCallback As Object
	Private mEvent As String
	Private mDebug As Boolean
	Private mChunkedUploadPos As Int
	Private mChunkedUploadLength As Int
	Private mChunkedUploadPath As String
	Private mChunkedUploadFilename As String
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize(CallbackModule As Object, EventName As String, AccessToken As String, debug As Boolean) 
	mCallback = CallbackModule
	mEvent = EventName
	mAccessToken = AccessToken
	mDebug = debug
End Sub


' API V1 (Dropbox CORE)
Sub GetDownloadLink(path As String)

	Dim job As HttpJob
	job.Initialize("downloadlink",Me)
	
			 	
	job.PostString(mApiV1url&"media/auto/"&path,Null)
	job.Tag = path
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
Sub GetShareFileLink(path As String)
	Dim job As HttpJob
	job.Initialize("sharefile",Me)
	job.Tag = path
	job.PostString(mApiV1url&"shares/auto/"&path,Null)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
Sub SaveURL(url As String, destpath As String)
	Log($"url="${url}""$)
	Dim job As HttpJob
	job.Initialize("saveurl",Me)
	job.Tag = url
	job.PostString(mApiV1url&"save_url/auto/"&destpath&"/?url="&url, "")
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
Sub GetURLJob(JobID As String)
	Log($"urljob="${JobID}""$)
	Dim Job As HttpJob
	Job.Initialize("saveurljob",Me)
	Job.Tag = JobID
	Job.PostString(mApiV1url&"save_url_job/"&JobID, "")
	Job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	Job.GetRequest.SetHeader("Content-Type", "application/json")
	Job.GetRequest.SetContentType("application/json")
	Job.GetRequest.SetContentEncoding("text/plain")
End Sub

Sub InitChunkUpload(localPath As String, localFile As String)
	Log($"InitChunkUpload="${localPath}, ${localFile}""$)
	mChunkedUploadPos = 0
	mChunkedUploadPath = localPath
	mChunkedUploadFilename = localFile
	Dim Job As HttpJob
	Job.Initialize("InitChunkUpload",Me)
	Job.Tag = File.Combine(mChunkedUploadPath,mChunkedUploadFilename)
	Dim input As InputStream = File.OpenInput(mChunkedUploadPath,mChunkedUploadFilename)
	mChunkedUploadLength = input.BytesAvailable
	Dim buffer(4096) As Byte 
	input.ReadBytes(buffer,mChunkedUploadPos,buffer.Length)
	' Data() as Byte
	Job.PostBytes(mApiV1urlcontent&"chunked_upload", buffer)
	Job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	Job.GetRequest.SetHeader("Content-Type", "application/json")
	Job.GetRequest.SetContentType("application/json")
	Job.GetRequest.SetContentEncoding("text/plain")
End Sub

Sub ContinueChunkUpload(localPath As String, localFile As String)
	Log($"InitChunkUpload="${localPath}, ${localFile}""$)
	'mChunkedUploadPos = 0
	Dim input As InputStream = File.OpenInput(mChunkedUploadPath,mChunkedUploadFilename)
	mChunkedUploadLength = input.BytesAvailable
	If mChunkedUploadPos < mChunkedUploadLength Then
		Dim buffer(4096) As Byte 
		input.ReadBytes(buffer,mChunkedUploadPos,buffer.Length)
		' Data() as Byte
		Dim Job As HttpJob
		Job.Initialize("ChunkUpload",Me)
		Job.Tag = File.Combine(mChunkedUploadPath,mChunkedUploadFilename)
		Job.PostBytes(mApiV1urlcontent&"chunked_upload", buffer)
		Job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
		Job.GetRequest.SetHeader("Content-Type", "application/json")
		Job.GetRequest.SetContentType("application/json")
		Job.GetRequest.SetContentEncoding("text/plain")
	End If
	

	'Job.Tag = File.Combine(localPath,localFile)
	'Dim input As InputStream = File.OpenInput(localPath,localFile)
	'mChunkedUploadLength = input.BytesAvailable
	'Dim buffer(4096) As Byte 
	'input.ReadBytes(buffer,mChunkedUploadPos,buffer.Length)
	' Data() as Byte
	'Job.PostBytes(mApiV1urlcontent&"chunked_upload", buffer)
	'Job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	'Job.GetRequest.SetHeader("Content-Type", "application/json")
	'Job.GetRequest.SetContentType("application/json")
	'Job.GetRequest.SetContentEncoding("text/plain")
End Sub


' API V2 Beta2
Sub GetCurrentAccount()
	Dim job As HttpJob
	job.Initialize("getcurrentaccount",Me)
	job.PostString(mApiV2url&"users/get_current_account",Null)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
Sub Listfolder(path As String)
	Dim job As HttpJob
	job.Initialize("listfolder",Me)
	job.Tag = path
	job.PostString(mApiV2url&"files/list_folder",$"{"path":"${path}"}"$)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
Sub GetMetadata(path As String)
	Dim job As HttpJob
	job.Initialize("getmetadata",Me)
	job.Tag = path
	job.PostString(mApiV2url&"files/get_metadata",$"{"path":"${path}"}"$)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
Sub Upload(localpath As String, localFilename As String, path As String)
	Dim job As HttpJob
	job.Initialize("fileupload",Me)
	job.Tag = path
	job.PostFile(mApiV2urlContent&"files/upload",localpath,localFilename)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	'job.GetRequest.SetHeader("Content-Type", "application/octet-stream")
	job.GetRequest.SetHeader("Dropbox-API-Arg", $"{"path":"${path}", "mode": "overwrite"}"$)
	job.GetRequest.SetContentType("application/octet-stream")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
Sub Download(path As String)
	Dim job As HttpJob
	job.Initialize("filedownload",Me)
	job.Tag = path
	job.Download(mApiV2urlContent&"files/download")
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Dropbox-API-Arg", $"{"path":"${path}"}"$)
	job.GetRequest.SetHeader("Accept","application/vnd.dropbox-cors-hack")
End Sub
Sub FileSearch(path As String, searchquery As String)
	Dim job As HttpJob
	job.Initialize("filesearch",Me)
	job.PostString(mApiV2url&"files/search",$"{"path":"${path}", "query": "${searchquery}"}"$)
	job.GetRequest.SetHeader("Authorization", "Bearer "&mAccessToken)
	job.GetRequest.SetHeader("Content-Type", "application/json")
	job.GetRequest.SetContentType("application/json")
	job.GetRequest.SetContentEncoding("text/plain")
End Sub
Sub JobDone(Job As HttpJob)
	If mDebug Then
		Log("JobDone("&Job.JobName&")")
	End If
  ProgressDialogHide
  If Job.Success Then
    Dim res As String
    res=Job.GetString
		If mDebug Then
  	  Log("JobName: " & Job.JobName)
    	If Job.JobName <> "filedownload" Then
				Log("Server response: " & res)
			End If
		End If
    Select Job.JobName
			Case "initchunkupload"
				Dim parser As JSONParser
				parser.Initialize(res)
				Dim root As Map = parser.NextObject
				Dim expires As String = root.Get("expires")
				Dim upload_id As String = root.Get("upload_id")
				Dim offset As Int = root.Get("offset")
				mChunkedUploadPos = offset
				ContinueChunkUpload(mChunkedUploadPath,mChunkedUploadFilename)
			Case "saveurljob"
				Dim parser As JSONParser
				parser.Initialize(res)
				Dim root As Map = parser.NextObject
				Dim status As String = root.Get("status")
				Dim JobID As String = root.Get("tag") 
			
			Case "saveurl"
				Dim parser As JSONParser
				parser.Initialize(res)
				Dim root As Map = parser.NextObject
				Dim status As String = root.Get("status")
				Dim saveurlJob As String = root.Get("job") 
				
			Case "filedownload"
				Dim dest As String = GetFilename(Job.Tag)
        Dim OutStream As OutputStream
        Log("DownloadReady: "&Job.Tag)
        OutStream = File.OpenOutput(File.DirRootExternal, dest, False) ' Job.Tag is read to set the Original Filename we specify earlier in the creation of the Job
        File. Copy2(Job.GetInputStream,OutStream) ' save the file
        OutStream.Close
        Log(Job.Tag&" are written to "&File.DirRootExternal) ' Write the Originalname to the log to see what happens ;-)
				
			Case "fileupload"
				Dim parser As JSONParser
				parser.Initialize(res)
				Dim root As Map = parser.NextObject
				Dim path As String = root.Get("path_lower")
				Dim rev As String = root.Get("rev")
				Dim name As String = root.Get("name")
				Dim size As Int = root.Get("size")
				Dim client_modified As String = root.Get("client_modified")
				Dim server_modified As String = root.Get("server_modified") 
				root.Put("Tag",Job.Tag)
				If SubExists(mCallback, mEvent & "_FileUpload") Then
					CallSub2(mCallback, mEvent & "_FileUpload", root)
				End If
				
			Case "downloadlink"
				Dim parser As JSONParser
				parser.Initialize(res)
				Dim root As Map = parser.NextObject
				Dim expires As String = root.Get("expires")
				Dim url As String = root.Get("url") 
				root.Put("Tag",Job.Tag)
				If SubExists(mCallback, mEvent & "_DownloadLink") Then
					CallSub2(mCallback, mEvent & "_DownloadLink", root)
				End If

			Case "sharefile"
				Dim parser As JSONParser
				parser.Initialize(res)
				Dim root As Map = parser.NextObject
				Dim expires As String = root.Get("expires")
				Dim visibility As String = root.Get("visibility")
				Dim url As String = root.Get("url") 
				root.Put("Tag",Job.Tag)
				If SubExists(mCallback, mEvent & "_ShareFile") Then
					CallSub2(mCallback, mEvent & "_ShareFile", root)
				End If
			Case "getmetadata"
				Dim parser As JSONParser
				parser.Initialize(res)
				Dim root As Map = parser.NextObject
				Dim path_lower As String = root.Get("path_lower")
				Dim rev As String = root.Get("rev")
				Dim name As String = root.Get("name")
				Dim size As Int = root.Get("size")
				Dim tag As String = root.Get(".tag")
				Dim client_modified As String = root.Get("client_modified")
				Dim server_modified As String = root.Get("server_modified") 
				root.Put("Tag",Job.Tag)
				
				If SubExists(mCallback, mEvent & "_MetaData") Then
					CallSub2(mCallback, mEvent & "_MetaData", root)
				End If
			Case "filesearch"
				Dim parser As JSONParser
				parser.Initialize(res)
				Dim root As Map = parser.NextObject
				Dim matches As List = root.Get("matches")
				For Each colmatches As Map In matches
					Dim match_type As Map = colmatches.Get("match_type")
					Dim match_type_tag As String = match_type.Get(".tag")
					Dim metadata As Map = colmatches.Get("metadata")
					Dim path_lower As String = metadata.Get("path_lower")
					Dim name As String = metadata.Get("name")
					Dim metatag As String = metadata.Get(".tag")
				Next
				Dim more As String = root.Get("more")
				Dim start As Int = root.Get("start") 
				If SubExists(mCallback, mEvent & "_FileSearch") Then
					CallSub2(mCallback, mEvent & "_FileSearch", root)
				End If
	    Case "listfolder"
				Dim parser As JSONParser
				parser.Initialize(res)
				Dim root As Map = parser.NextObject
				Dim cursor As String = root.Get("cursor")
				Dim has_more As String = root.Get("has_more")
				Dim entries As List = root.Get("entries")
				For Each colentries As Map In entries
					Dim path_lower As String = colentries.Get("path_lower")
					Dim name As String = colentries.Get("name")
					Dim tag As String = colentries.Get(".tag")
				Next 
				If SubExists(mCallback, mEvent & "_ListFolder") Then
					CallSub2(mCallback, mEvent & "_ListFolder", root)
				End If
	  Case "getcurrentaccount"
			Dim parser As JSONParser
			parser.Initialize(res)
			Dim root As Map = parser.NextObject
			Dim accreferral_link As String = root.Get("referral_link")
			Dim accaccount_id As String = root.Get("account_id")
			Dim accemail As String = root.Get("email")
			Dim acclocale As String = root.Get("locale")
			Dim accname As Map = root.Get("name")
			Dim accdisplay_name As String = accname.Get("display_name")
			Dim accfamiliar_name As String = accname.Get("familiar_name")
			Dim accsurname As String = accname.Get("surname")
			Dim accgiven_name As String = accname.Get("given_name")
			Dim is_paired As String = root.Get("is_paired")
			Dim account_type As Map = root.Get("account_type")
				Dim tag As String = account_type.Get(".tag")
			Dim country As String = root.Get("country") 
			If SubExists(mCallback, mEvent & "_CurrentAccount") Then
				CallSub2(mCallback, mEvent & "_CurrentAccount", root)
			End If
    End Select
  Else
    Log("JobError: " & Job.ErrorMessage)
		ToastMessageShow("Error: " & Job.ErrorMessage,True)
  End If
	Job.Release           
End Sub
Sub GetFilename(fullpath As String) As String
   Return fullpath.SubString(fullpath.LastIndexOf("/") + 1)
End Sub
